Loading...
--- xnu/xnu-344.49/iokit/Kernel/IOMultiMemoryDescriptor.cpp
+++ xnu/xnu-1228/iokit/Kernel/IOMultiMemoryDescriptor.cpp
@@ -1,16 +1,19 @@
/*
* Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
*
- * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1999-2003 Apple Computer, Inc. All Rights Reserved.
+ * @APPLE_OSREFERENCE_LICENSE_HEADER_START@
*
* This file contains Original Code and/or Modifications of Original Code
* as defined in and that are subject to the Apple Public Source License
* Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * compliance with the License. The rights granted to you under the License
+ * may not be used to create, or enable the creation or redistribution of,
+ * unlawful or unlicensed copies of an Apple operating system, or to
+ * circumvent, violate, or enable the circumvention or violation of, any
+ * terms of an Apple operating system software license agreement.
+ *
+ * Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this file.
*
* The Original Code and all software distributed under the License are
* distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
@@ -20,7 +23,7 @@
* Please see the License for the specific language governing rights and
* limitations under the License.
*
- * @APPLE_LICENSE_HEADER_END@
+ * @APPLE_OSREFERENCE_LICENSE_HEADER_END@
*/
#include <IOKit/IOLib.h>
@@ -90,7 +93,7 @@
IOMemoryDescriptor ** descriptors,
UInt32 withCount,
IODirection withDirection,
- bool asReference = false )
+ bool asReference )
{
//
// Create a new IOMultiMemoryDescriptor. The "buffer" is made up of several
@@ -121,7 +124,7 @@
IOMemoryDescriptor ** descriptors,
UInt32 withCount,
IODirection withDirection,
- bool asReference = false )
+ bool asReference )
{
//
// Initialize an IOMultiMemoryDescriptor. The "buffer" is made up of several
@@ -134,10 +137,19 @@
assert(descriptors);
assert(withCount);
- // Ask our superclass' opinion.
-
- if ( super::init() == false ) return false;
-
+ // Release existing descriptors, if any
+ if ( _descriptors )
+ {
+ for ( unsigned index = 0; index < _descriptorsCount; index++ )
+ _descriptors[index]->release();
+
+ if ( _descriptorsIsAllocated )
+ IODelete(_descriptors, IOMemoryDescriptor *, _descriptorsCount);
+ } else {
+ // Ask our superclass' opinion.
+ if ( super::init() == false ) return false;
+ }
+
// Initialize our minimal state.
_descriptors = 0;
@@ -264,9 +276,35 @@
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-IOPhysicalAddress IOMultiMemoryDescriptor::getPhysicalSegment(
- IOByteCount offset,
- IOByteCount * length )
+addr64_t IOMultiMemoryDescriptor::getPhysicalSegment64(
+ IOByteCount offset, IOByteCount * length )
+{
+ //
+ // This method returns the physical address of the byte at the given offset
+ // into the memory, and optionally the length of the physically contiguous
+ // segment from that offset.
+ //
+
+ assert(offset <= _length);
+
+ for ( unsigned index = 0; index < _descriptorsCount; index++ )
+ {
+ if ( offset < _descriptors[index]->getLength() )
+ {
+ return _descriptors[index]->getPhysicalSegment64(offset, length);
+ }
+ offset -= _descriptors[index]->getLength();
+ }
+
+ if ( length ) *length = 0;
+
+ return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+IOPhysicalAddress IOMultiMemoryDescriptor::getPhysicalSegment(
+ IOByteCount offset, IOByteCount * length )
{
//
// This method returns the physical address of the byte at the given offset